@@ -90,11 +90,16 @@ using namespace TagLib;
 /* Declaring the functions which should be accessible on the C side. */
 extern "C"
 {
+#define CAML_INTERNALS
 #include <caml/alloc.h>
 #include <caml/callback.h>
 #include <caml/fail.h>
 #include <caml/memory.h>
 #include <caml/signals.h>
+#include <caml/misc.h>
+#ifdef WIN32
+#include <caml/osdeps.h>
+#endif
 
 /* Use new thread syntax in a backward fashion. */
 #define caml_acquire_runtime_system caml_leave_blocking_section
@@ -176,7 +181,12 @@ CAMLprim value caml_taglib_file_new(value type, value name)
   CAMLparam2(name,type);
 
   File *f = NULL;
+
+#ifdef WIN32
+  const wchar_t *filename = caml_stat_strdup_to_os(String_val(name));
+#else
   char *filename = strdup(String_val(name));
+#endif
 
   if (filename == NULL)
     caml_raise_out_of_memory();
@@ -226,10 +236,7 @@ CAMLprim value caml_taglib_file_new(value type, value name)
   free(filename);
   caml_acquire_runtime_system();
 
-  if (f == NULL)
-    caml_raise_constant(*caml_named_value("taglib_exn_not_found"));
-
-  if (!f->isValid()) {
+  if (!(f && f->isValid())) {
     delete f;
     caml_raise_constant(*caml_named_value("taglib_exn_invalid_file"));
   }
@@ -306,7 +313,7 @@ CAMLprim value caml_taglib_tag_get_string(value t, value name)
   CAMLparam2(t,name);
   CAMLlocal1(ans);
   const Tag *tag = Taglib_tag_val(t) ;
-  char *s = String_val(name);
+  const char *s = String_val(name);
   String tmp = String::null;
 
   if (!strcmp(s,"title"))
@@ -333,7 +340,7 @@ CAMLprim value caml_taglib_tag_get_int(value t, value name)
 {
   CAMLparam2(t,name);
   const Tag *tag = Taglib_tag_val(t) ;
-  char *s = String_val(name);
+  const char *s = String_val(name);
   int tmp;
 
     if (!strcmp(s,"year"))
@@ -393,9 +400,9 @@ CAMLprim value caml_taglib_file_set_properties(value f, value properties)
   CAMLlocal1(caml_values);
   File *file = Taglib_file_val(f);
   PropertyMap props;
-  char *caml_key;
+  const char *caml_key, *caml_val;
   StringList *values;
-  String *key;
+  String *key, *val;
   int i,j;
   
   for (i = 0; i < Wosize_val(properties); i++) {
@@ -405,12 +412,15 @@ CAMLprim value caml_taglib_file_set_properties(value f, value properties)
     key = new String(caml_key, String::UTF8);
     values = new StringList();
     for (j = 0; j < Wosize_val(caml_values); j++) {
-      values->append(String_val(Field(caml_values,j)));
+      caml_val = String_val(Field(caml_values,j));
+      val = new String(caml_val, String::UTF8);
+      values->append(*val);
     }
 
     props.insert(*key, *values);
 
-    delete key, values;
+    delete key;
+    delete values;
   }
 
   // TODO: Catch this:
@@ -431,7 +441,7 @@ CAMLprim value caml_taglib_tag_set_string(value t, value name, value v)
 {
   CAMLparam3(t,name, v);
   Tag *tag = Taglib_tag_val(t) ;
-  char *s = String_val(name);
+  const char *s = String_val(name);
   const char *x = String_val(v) ;
 
     if (!strcmp(s,"title"))
@@ -454,7 +464,7 @@ CAMLprim value caml_taglib_tag_set_int(value t, value name, value v)
 {
   CAMLparam3(t,name, v);
   Tag *tag = Taglib_tag_val(t) ;
-  char *s = String_val(name);
+  const char *s = String_val(name);
   int x = Int_val(v) ;
 
     if (!strcmp(s,"year"))
@@ -471,7 +481,7 @@ CAMLprim value caml_taglib_audioproperties_get_int(value p, value name)
 {
   CAMLparam2(p,name);
   const AudioProperties *prop = Taglib_audioproperties_val(p) ;
-  char *s = String_val(name);
+  const char *s = String_val(name);
   int tmp;
 
     if (!strcmp(s,"length"))
@@ -554,7 +564,7 @@ CAMLprim value caml_taglib_id3v2_parse_tag(value t, value h)
 {
   CAMLparam2(t,h);
   myId3v2 *tag = Id3v2_tag_val(t);
-  char *s = String_val(h);
+  const char *s = String_val(h);
   TagLib::uint size = ID3v2::Header::size();
 
   tag->doParse(ByteVector(s+size,caml_string_length(h)-size));
@@ -569,8 +579,11 @@ CAMLprim value caml_taglib_id3v2_render(value t)
   ByteVector r = tag->render();
 
   ret = caml_alloc_string(r.size());
+#ifdef CAML_SAFE_STRING
+  memcpy(Bytes_val(ret),r.data(),r.size());
+#else
   memcpy(String_val(ret),r.data(),r.size());
-
+#endif
   CAMLreturn(ret);
 }
 
